package de.hpi.searchengines;
import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import org.apache.lucene.analysis.Analyzer;
import org.apache.lucene.analysis.SimpleAnalyzer;
import org.apache.lucene.document.Document;
import org.apache.lucene.document.Field;
import org.apache.lucene.index.CorruptIndexException;
import org.apache.lucene.index.IndexWriter;
import org.apache.lucene.index.IndexWriterConfig;
import org.apache.lucene.queryParser.ParseException;
import org.apache.lucene.queryParser.QueryParser;
import org.apache.lucene.search.IndexSearcher;
import org.apache.lucene.search.Query;
import org.apache.lucene.search.ScoreDoc;
import org.apache.lucene.search.TopScoreDocCollector;
import org.apache.lucene.store.Directory;
import org.apache.lucene.store.LockObtainFailedException;
import org.apache.lucene.store.RAMDirectory;
import org.apache.lucene.util.Version;


public class LuceneExercise {
	public static final String ATTRIBUTE_NAME = "title";
	
	Directory index; // store the index
	QueryParser queryParser; // parse and analyze the query
	IndexSearcher searcher;  // search the index with a query
	TopScoreDocCollector collector; // collect results
	
	/**
	 * Adds a document (here: string) to the index.
	 * 
	 * @param indexWriter writes the index
	 * @param value the value to index
	 * @throws IOException
	 */
	private void addDocument(IndexWriter indexWriter, String value) throws IOException {
		Document doc = new Document();
		doc.add(new Field(ATTRIBUTE_NAME, value, Field.Store.YES, Field.Index.ANALYZED));
		indexWriter.addDocument(doc);
	}

	/**
	 * Index a list of strings
	 * 
	 * @param stringList list of strings
	 */
	public void index(List<String> stringList) {
		
		// where to store the index
		index = new RAMDirectory();
		
		// how to analyze the input text
		Analyzer analyzer = new SimpleAnalyzer(Version.LUCENE_31);



		// configuration of index writer
		IndexWriterConfig conf = new IndexWriterConfig(Version.LUCENE_31, analyzer);
		// writes the index to the directory
		IndexWriter indexWriter;
		try {
			indexWriter = new IndexWriter(index, conf);
			
			// add documents to the index
			for (String string : stringList)
				addDocument(indexWriter, string);
			indexWriter.close();
		} catch (CorruptIndexException e) {
			e.printStackTrace();
		} catch (LockObtainFailedException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}



		// query preparation
		// query parser uses same analyzer as index writer
		queryParser = new QueryParser(Version.LUCENE_31, ATTRIBUTE_NAME, analyzer);
		try {
			searcher = new IndexSearcher(index, true);
		} catch (CorruptIndexException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}

	}

	/**
	 * Query the index with a string
	 * 
	 * @param queryString query string
	 * @param maxHits max number of hits
	 */
	public void query(String queryString, int maxHits) {

		// print query
		System.out.println("\nQuery:\t"+queryString);

		Query q;
		try {
			// prepare query
			q = queryParser.parse(queryString);
			collector = TopScoreDocCollector.create(maxHits, true);
			
			// search index
			searcher.search(q, collector);

			// collect results
			ScoreDoc[] hits = collector.topDocs().scoreDocs;
			
			// print results
			System.out.println("Results:");
			for (ScoreDoc scoreDoc : hits) {
				Document doc = searcher.doc(scoreDoc.doc);
				System.out.println("\t" + doc.get("title"));
			}

		} catch (ParseException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}

	}

	public static void main(String[] args) {
		// read movie names to be indexed
		String fileName = "MovieTitles.txt";
		
		List<String> stringList = new ArrayList<String>();
		try {
			BufferedReader input =  new BufferedReader(new FileReader(fileName));
			String line = null; 
			while (( line = input.readLine()) != null){
				stringList.add(line);
			}
		} catch (IOException e) {
			e.printStackTrace();
		}


		// create index
		LuceneExercise lsc = new LuceneExercise();
		lsc.index(stringList);

		int maxHits = 10;
		
		// answer queries!
		
		// expected result: Indiana Jones and the Last Crusade
		// exact match
		String query = "indiana";
		lsc.query(query, maxHits);
		// expected result: Heat
		// stop words
		query = "the AND heat";
		lsc.query(query, maxHits);
		// expected result: Goodfellas
		// stemming
		query = "Goodfella";
		lsc.query(query, maxHits);
		// expected result: Inception
		// single term suggestion
		query = "inceptoin";
		lsc.query(query, maxHits);
		// expected result: Groundhog Day
		// phrase suggestion
		query = "groundhg AND day";
		lsc.query(query, maxHits);

	}


}
